<?php
    class SystemUpdateRead
    {
        private $Configuration = null;
        private $BackupDirectory = null;
        
        /**
         * Initialize file path for update configuration
         * @param string $file Update file path
         */
        public function __construct($file)
        {
            if(file_exists($file))
            {
                $this->Configuration = json_decode( file_get_contents($file), true );
            }
            else
                throw new Exception("Configuration file does not exist.");
        }
        
        /**
         * Retrieving information about update
         * @return array Date and changelog
         */
        public function info()
        {
            $return = $this->Configuration['info'];
            if($return)
            {
                $return['changelog'] = base64_decode($return['changelog']);
                $return['sql'] = base64_decode($return['sql']);
                
                foreach($this->Configuration['files'] as $file)
                {
                    $return['files'][] = $file['path'];
                }
                return $return;
            }
            return false;
        }
        
        /**
         * Checking if directories are ready to write
         * @param string $changeDir Temporary change dir for root
         * @return bool
         */
        public function test($changeDir = null)
        {
            $return = true;
            foreach($this->Configuration['files'] as $file)
            {
                $status = $this->_isParentWritable($changeDir.$file['path']);
                if(!$status['status'])
                {
                    $return = false;
                    $this->error[] = $status['path'];
                }
            }
            
            return $return;
        }
        
        /**
         * Create backup for replacing files
         * @param string $changeDir Temporary change dir for root
         * @return bool Returns True or False
         */
        public function makeBackup($changeDir = null)
        {    
            // Scan for existing files and copy them to separate directory
            $tempDirectory = "backup_".date("Y-m-d-H-i-s")."-".md5(time()).'/';
            $this->BackupDirectory = $tempDirectory;
            
            if(mkdir($changeDir.$tempDirectory, 0777))
            {
                foreach($this->Configuration['files'] as $file)
                {
                    if(file_exists($changeDir.$file['path']))
                    {
                        if(!file_exists( $changeDir.$tempDirectory.pathinfo($file['path'], PATHINFO_DIRNAME) ))
                            mkdir( $changeDir.$tempDirectory.pathinfo($file['path'], PATHINFO_DIRNAME), 0777, true );
                        
                        if(!copy($changeDir.$file['path'], $changeDir.$tempDirectory.$file['path'])) return false;
                    }
                }
            }
            else
                return false;
            
            // Return True if everything was OK.
            return true;
        }
        
        /**
         * Restore backup
         * @param string $changeDir Temporary change dir for root
         * @return bool Returns True or False
         */
        public function restoreBackup($changeDir = null)
        {            
            // Scan for existing files and copy them to separate directory
            $tempDirectory = $this->BackupDirectory;
            
            foreach($this->Configuration['files'] as $file)
            {
                if(file_exists($tempDirectory.$file['path']))
                    copy($tempDirectory.$file['path'], $changeDir.$file['path']);
            }
            
            // Return True if everything was OK.
            return true;
        }
        
        /**
         *  Start create/update files
         *  @return bool Returns True or False
         */
        public function update($changeDir = null)
        {
            foreach($this->Configuration['files'] as $file)
            {
                $content = base64_decode( $file['content'] );
                if( !file_exists(pathinfo($changeDir.$file['path'], PATHINFO_DIRNAME)))
                    mkdir(pathinfo($changeDir.$file['path'], PATHINFO_DIRNAME), 0777, true);
                
                if(empty($content))
                    touch($changeDir.$file['path']);
                else if( !file_put_contents($changeDir.$file['path'], $content) )
                {
                    return false;
                }
            }
            
            $sql = base64_decode($this->Configuration['info']['sql']);
            $sql = explode(";", $sql);
            foreach($sql as $query)
                mysql_query($query);
                
            return true;
        }
        
        private function _isParentWritable($path)
        {
            if(file_exists($path))
            {
                return [
                        'status'    => is_writable($path),
                        'path'      => $path
                ];
            }
            else
            {
                return $this->_isParentWritable( pathinfo($path, PATHINFO_DIRNAME) );
            }
        }
    }
?>